CSRF_KEY = 'csrfmiddlewaretoken'
ELAB_URL = 'https://us.elabjournal.com/'
# ELAB_URL = 'https://elab.msl.ubc.ca/'
ELAB_API = '%sapi/v1/' % ELAB_URL

SERVER_API_VER = 'd1'
SERVER_URL = 'http://127.0.0.1:8001/api/%s/' % SERVER_API_VER
SERVER_DB_PATH = 'limes_server/db'

SHAMWOW_URL = 'shamwow.microbiology.ubc.ca'
            